/*
 * jWinkeyServer, a cwdeamon replacement that interfaces to Winkeyer by K1EL.
 * Copyright (C) 2012  Andy Summers, G4KNO (g4kno.mail@gmail.com).
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package winkeyserver;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Properties;

/**
 * Class Settings.
 * Handles the storing/retrieving of user preferences to/from a configuration file.
 */
public class Settings {

    //private static String fileName = "winkeyserver.conf";
    private static String fileName = "resources/winkeyserver.conf";
    private static File file;
    
    public static String CWDAEMON_PORT;
    public static String SERIAL_PORT;
    public static String HOST_SETS_SPEED;
    public static String HOST_WPM;
    public static String WINKEY_MIN_WPM;
    public static String WINKEY_WPM_RANGE;
    public static String WINKEY_MODE;
    public static String WINKEY_SIDETONE;
    public static String WINKEY_WEIGHT;
    public static String WINKEY_PTT_LEAD;
    public static String WINKEY_PTT_TAIL;
    public static String WINKEY_EXTN1;
    public static String WINKEY_KEY_COMP;
    public static String WINKEY_WPM_FARNS;
    public static String WINKEY_PADDLE_SWITCHPOINT;
    public static String WINKEY_DAH_DIT_RATIO;
    public static String WINKEY_PIN_CONFIG;
    public static String CWDAEMON_CONTROLS_PTT;
    public static String POSITION_X;
    public static String POSITION_Y;
    public static String MAX_TEXTLINES;
    public static String DEBUG;

    //A singleton is created at JVM load.
    private static Settings s = new Settings();

    //Default private contructor can't be instantiated externally.
    private Settings() {
        try {
            file = new File("winkeyserver.conf");
            // Create this file if it doesn't already exist.
            // If run from the shell it will be dropped where it was started.
            // If run from the file manager it will be dropped in the home directory.
            file.createNewFile();            
        } catch (Exception e) {
            System.err.println(e.toString());
        }
        readIni();
    }

    //Delegate public method to provide static access to the class.
    public static Settings getInstance() {
        //Always return the instance of the class created at JVM load.
        return s;
    }

    //Private method to read user settings from .ini file.
    private static void readIni(){
        try {
            Properties p = new Properties();
            p.load(new FileInputStream(file));
            //p.list(System.out);

            
            CWDAEMON_PORT = p.getProperty("CWDAEMON_PORT", "6789");
            SERIAL_PORT = p.getProperty("SERIAL_PORT", "ttyUSB0");
            HOST_SETS_SPEED = p.getProperty("HOST_SETS_SPEED", "true");
            HOST_WPM = p.getProperty("HOST_WPM", "14");
            WINKEY_MIN_WPM = p.getProperty("WINKEY_MIN_WPM", "10");
            WINKEY_WPM_RANGE = p.getProperty("WINKEY_WPM_RANGE", "0E");
            WINKEY_MODE = p.getProperty("WINKEY_MODE", "00");
            WINKEY_SIDETONE = p.getProperty("WINKEY_SIDETONE", "05");
            WINKEY_WEIGHT = p.getProperty("WINKEY_WEIGHT", "32");
            WINKEY_PTT_LEAD = p.getProperty("WINKEY_PTT_LEAD", "00");
            WINKEY_PTT_TAIL = p.getProperty("WINKEY_PTT_TAIL", "00");
            WINKEY_EXTN1 = p.getProperty("WINKEY_EXTN1", "00");
            WINKEY_KEY_COMP = p.getProperty("WINKEY_KEY_COMP", "00");
            WINKEY_WPM_FARNS = p.getProperty("WINKEY_WPM_FARNS", "0A");
            WINKEY_PADDLE_SWITCHPOINT = p.getProperty("WINKEY_PADDLE_SWITCHPOINT", "32");
            WINKEY_DAH_DIT_RATIO = p.getProperty("WINKEY_DAH_DIT_RATIO", "32");
            WINKEY_PIN_CONFIG = p.getProperty("WINKEY_PIN_CONFIG", "0F");
            CWDAEMON_CONTROLS_PTT = p.getProperty("CWDAEMON_CONTROLS_PTT", "false");
            POSITION_X = p.getProperty("POSITION_X", "0");
            POSITION_Y = p.getProperty("POSITION_Y", "0");
            MAX_TEXTLINES = p.getProperty("MAX_TEXTLINES", "100");
            DEBUG = p.getProperty("DEBUG", "false");


        } catch (Exception e) {
            System.err.println(e);
        }
    }

    //Public method to write user settings to .ini file.
    public static void writeIni() {
        try{
            Properties p = new Properties();
            
            
            p.put("CWDAEMON_PORT", CWDAEMON_PORT);
            p.put("SERIAL_PORT", SERIAL_PORT);
            p.put("HOST_SETS_SPEED", HOST_SETS_SPEED);
            p.put("HOST_WPM", HOST_WPM);
            p.put("WINKEY_MIN_WPM", WINKEY_MIN_WPM);
            p.put("WINKEY_WPM_RANGE", WINKEY_WPM_RANGE);
            p.put("WINKEY_MODE", WINKEY_MODE);
            p.put("WINKEY_SIDETONE", WINKEY_SIDETONE);
            p.put("WINKEY_WEIGHT", WINKEY_WEIGHT);
            p.put("WINKEY_PTT_LEAD", WINKEY_PTT_LEAD);
            p.put("WINKEY_PTT_TAIL", WINKEY_PTT_TAIL);
            p.put("WINKEY_EXTN1", WINKEY_EXTN1);
            p.put("WINKEY_KEY_COMP", WINKEY_KEY_COMP);
            p.put("WINKEY_WPM_FARNS", WINKEY_WPM_FARNS);
            p.put("WINKEY_PADDLE_SWITCHPOINT", WINKEY_PADDLE_SWITCHPOINT);
            p.put("WINKEY_DAH_DIT_RATIO", WINKEY_DAH_DIT_RATIO);
            p.put("WINKEY_PIN_CONFIG", WINKEY_PIN_CONFIG);
            p.put("CWDAEMON_CONTROLS_PTT", CWDAEMON_CONTROLS_PTT);
            p.put("POSITION_X", POSITION_X);
            p.put("POSITION_Y", POSITION_Y);
            p.put("MAX_TEXTLINES", MAX_TEXTLINES);
            p.put("DEBUG", DEBUG);
            

            FileOutputStream out = new FileOutputStream(file);
            p.store(out, "/* properties updated */");
        } catch (Exception e) {
            System.out.println(e);
        }
    }

    

}
